package com.biggs;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.Adler32;

import org.apache.commons.compress.archivers.zip.UnrecognizedExtraField;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.archivers.zip.ZipFile;
import org.apache.commons.compress.archivers.zip.ZipLong;
import org.apache.commons.compress.archivers.zip.ZipShort;

/**
 * 
 * @author Damien Biggs
 * @email damien.biggs@gmail.com
 * Utility class for injecting a adler32 checksum into every file in the input zip file.
 * The updated files are stored in the output file.
 * Released under zlib/libpng license.
 */
public class Adler32Injector {

	public static byte[] getBytesFromStrean(InputStream is, long length, String name) throws IOException {


		// Create the byte array to hold the data
		byte[] bytes = new byte[(int) length];

		// Read in the bytes
		int offset = 0;
		int numRead = 0;
		while (offset < bytes.length
				&& (numRead = is.read(bytes, offset, bytes.length - offset)) >= 0) {
			offset += numRead;
		}
		// Ensure all the bytes have been read in
		if (offset < bytes.length) {
			throw new IOException("Could not completely read from stream " + name);
		}

		// Close the input stream and return bytes
		is.close();
		return bytes;
	}

	public static void main(String arg[]) {
		try {
			if (arg.length != 2)
			{
				System.out.println("Usage: java Adler32Injector inputZipFilePath OutputZipFilePath");
				System.exit(1);
			}
			ZipFile zippedIn = new ZipFile(arg[0]);
			ZipArchiveOutputStream zippedOut = new ZipArchiveOutputStream(new File(arg[1]));
			zippedOut.setMethod(ZipArchiveOutputStream.DEFLATED);
			Enumeration<ZipArchiveEntry> entries = zippedIn.getEntries();
			Adler32 adlerChecksum = new Adler32();
			while (entries.hasMoreElements())
			{
				ZipArchiveEntry entry = entries.nextElement();
				InputStream input = zippedIn.getInputStream(entry);
				byte[] content = getBytesFromStrean(input, entry.getSize(), entry.getName());
				adlerChecksum.reset();
				// checksum is calculated on the uncompressed bytes of the file in question
				adlerChecksum.update(content);

				// adds the adler32 checksum to the zip file
				UnrecognizedExtraField adlerField = new UnrecognizedExtraField();
				adlerField.setHeaderId(new ZipShort(0xdada));
				adlerField.setLocalFileDataData(new ZipLong(adlerChecksum.getValue()).getBytes());
				entry.addExtraField(adlerField);

				zippedOut.putArchiveEntry(entry);
				zippedOut.write(content);
				zippedOut.closeArchiveEntry();				
			}
			zippedIn.close();
			zippedOut.close();
			System.out.println("Converted " + arg[0] + " to " + arg[1]);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
